package com.wulfson.livescore;


import com.illposed.osc.*;
import java.awt.geom.Rectangle2D;
import java.net.SocketException;
import java.text.AttributedString;
import java.util.HashMap;
import java.util.Timer;

public class LSOscController implements OSCListener
{

	OSCPortIn _receiver;
	LSScore _score;
	
	public LSOscController(LSScore score, int port) throws SocketException
	{
		_score = score;
		_receiver = new OSCPortIn(port);
		System.out.println(port);
        _receiver.addListener("/livescore", this);
        _receiver.startListening();	
		
	}
	
    public void acceptMessage(java.util.Date time, OSCMessage message)
    {
    	//we don't want this thread to crash
    	try
    	{
    		this.handleMessage(time, message);
    	}
    	catch(Exception e)
    	{
    		System.out.println("Exception in LSOscController: " + e.getMessage());
    	}
    }
    public void handleMessage(java.util.Date time, OSCMessage message)
    {
    	HashMap args;
    	
    	//System.out.println("got msg");
    	args=this.convertArrayToHashMap(message);

    	if(args.get("cmd").equals("notate"))
    	{
    		this.cmdNotate(args);
    	}
    	else if(args.get("cmd").equals("clear"))
    	{
    		this.cmdClear(args);
    	}    	
    	else if(args.get("cmd").equals("conductor"))
    	{
    		this.cmdConductor(args);
    	}
    	else if(args.get("cmd").equals("title"))
    	{
    		this.cmdTitle(args);
    	}
    	else if(args.get("cmd").equals("textbox"))
    	{
    		this.cmdTextBox(args);
    	}
    	else if(args.get("cmd").equals("cleartextboxes"))
    	{
    		this.cmdClearTextBoxes(args);
    	}
    }

    public void cmdNotate(HashMap args)
    {
    	Number value;
    	int part=0;
    	float time=0;
    	int pitch=60;
    	int dynamic=0;
    	float duration=0;
    	String text=null;
    	String instruction=null;
    	LSNote note;
    	LSNote lastNote;
    	LSStaff staff;
    	LSPosition position;
    	LSPage page;
    	LSBeamGroup beamGroup;
    	float xdistance;
    	
    	//TODO: catch nullPointerExceptions...
    	value=(Number)args.get("time"); if(value != null) { time=value.floatValue(); }
    	value=(Number)args.get("duration"); if(value != null) { duration=value.floatValue(); }
    	value=(Number)args.get("pitch"); if(value != null) { pitch=value.intValue(); }
    	value=(Number)args.get("dynamic"); if(value != null) { dynamic=value.intValue(); }
    	value=(Number)args.get("part"); if(value != null) { part=value.intValue(); }
    	text=(String)args.get("text");
    	instruction=(String)args.get("instruction");
    	
    	page = _score.getPage();
    	position = page.getPositionForTime(time);
    	staff=(LSStaff)((LSSystem)page.getSystems().get(position.getSystemIndex())).getStaves().get(part);

    	//Get last note in case beaming is needed
    	//TODO need better way to get previous note!!!!!!!
    	if(staff.getNotes().size() > 0)
    	{
    		lastNote = (LSNote)staff.getNotes().get(staff.getNotes().size() - 1);
    	}
    	else
    	{
    		lastNote = null;
    	}

    	//Make a note 
    	note = new LSNote(staff, position.getX(), 0, 0);
    	note.setPitch(pitch);
    	note.setOnset(time);
    	note.setDuration(duration);
    	note.setDurationBeamLength(duration * 30f); //TODO constant. redundant
    	note.setDynamic(dynamic);
    	note.setText(text);
    	note.setInstruction(instruction);
    	if(duration == 0)
    	{
    		note.setHasStem(false);
    	}
    	
    	//Natural?
    	if(note.getAccidental() == LSNote.NOACCIDENTAL)
    	{
    		for(LSNote trailNote : staff.getNoteTrail()) //Java 1.5 foreach feature
    		{
    			if(trailNote.getSteps() == note.getSteps() &&
 					trailNote.getAccidental() == LSNote.SHARP) //TODO add flats
				{
					note.setAccidental(LSNote.NATURAL);
				}
    		}
    	}
    		
        	
    	//Do we need to beam? !!!!!!! need better method for score view. 
    	if(lastNote != null)
    	{
    		//xdistance = note.getXpos() - staff.getLastNoteEndBeforeX(note.getXpos());
    		xdistance = note.getXpos() - (lastNote.getXpos() + lastNote.getDurationBeamLength());
	    	if (xdistance < 15f) //TODO 15. TODO bad way of checking prev note X. 
			{
	    		//note.setText(new Float(xdistance).toString());
				beamGroup = lastNote.getBeamGroup();
				if (beamGroup == null)
				{
					beamGroup = new LSBeamGroup();
					beamGroup.setHasBeam(false);
					beamGroup.addNote(lastNote);
				}
				beamGroup.addNote(note);
				staff.addBeamGroup(beamGroup);
				beamGroup.beam();
			}
    	}
    	staff.addNote(note);
    	
    	//Schedule the note erasure. (15 seconds)
    	new Timer().schedule(new LSNoteExpire(note), (long)((note.getOnset() + note.getDuration() - _score.getTime()) * 1000) + 15000); //46000 ); //TODO

    	//Add to trail
    	staff.addToNoteTrail(note);
    	if(staff.getNoteTrail().size() == 10) //TODO: trail size setting
    	{
    		staff.getNoteTrail().remove(0);
    	}
    	
    	//Blank out next line. TODO this should happen when conductor jumps system. not here. 
    	//Actually, I'm going to try expiring notes instead. 3/31/2008
    	/*
    	staff=(LSStaff)((LSSystem)page.getSystems().
    			get((position.getSystemIndex() + 1) % page.getSystems().size()))
    			.getStaves().get(0);
    	if(staff.getNotes().size() > 0)
    	{
    		staff.clear();
    	}
    	*/
    	
    	//_score.paintPageImage(); //TODO: maybe this should be a method on Page instead? 
    }

    public void cmdClear(HashMap args)
    {
    	Number value;
    	int clef;
    	int lines;
    	
    	//look for optional clef argument. TODO should this be seperate osc command?
    	value=(Number)args.get("clef");
    	if(value != null)
    	{
    		clef=value.intValue();
    		_score.getParts().get(0).setClef(clef);
    	}
    	value=(Number)args.get("lines");
    	if(value != null)
    	{
    		lines = value.intValue();
    		_score.getParts().get(0).setLines(lines);
    	}
    	_score.initNotation(_score.getSize());
    	_score.repaint();
    }
    public void cmdConductor(HashMap args)
    {
    	double time;
    	
    	time=((Number)args.get("time")).doubleValue();
    	//System.out.println("Was: " + _score.getTime() + ", now: " + time);
    	_score.setTime(time);
    	//_score.repaint(); //TODO: leave repaint to conductor thread?
    }
    
    public void cmdTitle(HashMap args)
    {
    	Number value;
    	String title;
    	int titleSize=30;
    	
    	title=(String)args.get("title");
    	value=(Number)args.get("titleSize"); if(value != null) { titleSize=value.intValue(); }
       	_score.getPage().setTitle(title); //TODO put title on score
       	_score.getPage().setTitleSize(titleSize);       	
    	_score.repaint(); //TODO: leave repaint to conductor thread?
    }
    
    public void cmdTextBox(HashMap args)
    {
    	LSTextBox box;
    	String header;
    	String text;

    	header=(String)args.get("header");
    	text=(String)args.get("text");
    	box=new LSTextBox();
    	box.setBox(new Rectangle2D.Float(200,100,400,350));
    	box.setHeader(header);
    	box.setText(new AttributedString(text));
    	_score.addTextBox(box);
    	
    	_score.repaint(); //TODO: leave repaint to conductor thread?
    }
    public void cmdClearTextBoxes(HashMap args)
    {
    	_score.clearTextBoxes();    	
    	_score.repaint(); //TODO: leave repaint to conductor thread?
    }
        
    public HashMap convertArrayToHashMap(OSCMessage message)
    {
    	HashMap map;
    	Object key,value;
    	int idx;
    	
    	Object[] args;
    	
    	map=new HashMap();
    	args=message.getArguments();
    	idx=0;
    	while(idx<message.getArguments().length)
    	{
    		key=args[idx];
    		value=args[idx+1];
    		map.put(key,value);
    		idx=idx+2;
    	}
    	return map;
    }

    //TODO: delete. no longer used. 
    public float getFloatFromNumber(Object number)
    {
	    if(number.getClass() == Integer.class)
		{
			return (float)((Integer)number).intValue();
		}
		else if(number.getClass() == Float.class)
		{
			return ((Float)number).floatValue();
		}
	    return 0;
    }

    //Accessors
	public LSScore getscore()
	{
		return _score;
	}

	public void setScore(LSScore score)
	{
		_score = score;
	}
    
}


