package com.wulfson.livescore;
//
//  LSDocument.java
//  LiveScore
//
//  Created by harris wulfson on 7/26/07.
//  Copyright 2007. All rights reserved.
//

//package

import java.util.*;
import java.awt.*;


public class LSPage
{
	protected float _space;
	protected Font _font;
	protected ArrayList<LSSystem> _systems;
	protected Dimension _pageSize;
	protected float _leftMargin;
	protected float _rightMargin;
	protected float _leftXInset;
	protected float _rightXInset;
	protected String _title;
	protected int _titleSize;
	
	
	
	public LSPage()
	{
		LSTextBox text;
		_systems=new ArrayList<LSSystem>();
		_space=5;
		_leftMargin = 30; //default
		_rightMargin = 30;
		_leftXInset = 35; //space for clef
		_rightXInset = 15;
		
		_title = "LiveScore";
		_titleSize = 30;
	}
	
	//doesn't clear notes off. it actually erases the systems from the page
	public void clear()
	{
		_systems=new ArrayList<LSSystem>(); //wipe all systems
	}
	
	public void paint(Graphics g)
	{
		Iterator i;
		LSSystem aSystem;
		
		//TODO move to frame?
		_font = new Font("Maestro", Font.PLAIN, (int)(_space * 4.0f));
		g.setFont(new Font("Helvetica", Font.PLAIN, _titleSize));
		g.drawString(_title, (int)_leftMargin, 10 + _titleSize);

		
		i=_systems.iterator();
		while(i.hasNext()) //TODO use for each
		{
			aSystem=(LSSystem)i.next();
			aSystem.paint(g);
		}
				
	}
	
	public float getSpace()
	{
		return _space;
	}

	public void setSpace(float aSpace)
	{
		_space = aSpace;
	}

	public Font getFont()
	{
		return _font;
	}

	public void setFont(Font aFont)
	{
		_font = aFont;
	}

	public ArrayList getSystems()
	{
		return _systems;
	}

	//add to ArrayList of Systems
	public void addSystem(LSSystem aSystem)
	{
		_systems.add(aSystem);
		aSystem.setPage(this);
	}

	public Dimension getPageSize()
	{
		return _pageSize;
	}

	public void setPageSize(Dimension aPageSize)
	{
		_pageSize = aPageSize;
	}

	public float getLeftMargin()
	{
		return _leftMargin;
	}
	public void setLeftMargin(float leftMargin)
	{
		_leftMargin = leftMargin;
	}

	public float getRightMargin()
	{
		return _rightMargin;
	}

	public void setRightMargin(float rightMargin)
	{
		_rightMargin = rightMargin;
	}
	public float getLeftXInset()
	{
		return _leftXInset;
	}

	public void setLeftXInset(float leftXInset)
	{
		_leftXInset = leftXInset;
	}

	public float getRightXInset()
	{
		return _rightXInset;
	}

	public void setRightXInset(float rightXInset)
	{
		_rightXInset = rightXInset;
	}
	public String getTitle()
	{
		return _title;
	}
	public void setTitle(String title)
	{
		_title = title;
	}
	public int getTitleSize()
	{
		return _titleSize;
	}
	public void setTitleSize(int titleSize)
	{
		_titleSize = titleSize;
	}

	//Calculations
	public LSPosition getPositionForTime(double time)
	{
		LSPosition position;
		int realSystemIndex;
		int systemIndex;
		float timeInSystem;
		float x;
		float pps; //points per second.
		float lol; //length of line, in seconds
		
		position = new LSPosition(); //basically a struct
		pps = 30f; //TODO
		lol = (_pageSize.width - _leftMargin - _rightMargin - _leftXInset - _rightXInset) / pps; //TODO
		//System.out.println(lol);
		realSystemIndex = (int)(time / lol);
		systemIndex = realSystemIndex % _systems.size();
		timeInSystem = (float)(time - (realSystemIndex * lol));
		x = (timeInSystem * pps) + _leftMargin + _leftXInset;
		//put into data structure
		position.setRealSystemIndex(realSystemIndex);
		position.setSystemIndex(systemIndex);
		position.setTimeInSystem(timeInSystem);
		position.setX(x);
		
		return position;
	}
	public LSPosition getAdditionalConductorPositionForPosition(LSPosition position)
	{
		LSPosition newPosition;
		float pps;
		float lol;
		float x;
		
		pps = 30f; //TODO
		lol = (_pageSize.width - _leftMargin - _rightMargin - _leftXInset - _rightXInset) / pps; //TODO

		newPosition = new LSPosition();
		if((position.getTimeInSystem() < (_rightXInset / pps)) && (position.getRealSystemIndex() > 0)) //finish out the previous system
		{
			newPosition.setRealSystemIndex(position.getRealSystemIndex() - 1);
			newPosition.setTimeInSystem(lol + position.getTimeInSystem());
		}
		else if(position.getTimeInSystem() > lol - (_leftXInset / pps))
		{
			newPosition.setRealSystemIndex(position.getRealSystemIndex() + 1);
			newPosition.setTimeInSystem(position.getTimeInSystem() - lol);
		}
		else
		{
			return null;
		}
		newPosition.setSystemIndex(newPosition.getRealSystemIndex() % _systems.size());
		newPosition.setX((newPosition.getTimeInSystem() * pps) + _leftMargin + _leftXInset);
		newPosition.setTime(position.getTime());
		return newPosition;
	}

	//Factories
	public LSSystem newSystem(float yoffset)
	{
		LSSystem system;
		
		system = new LSSystem(yoffset);
		system.setPage(this);
		system.setLeftX(_leftMargin);
		system.setRightX(_pageSize.width - _rightMargin);
		this.addSystem(system);
		
		return system;
	}
	
}