package com.wulfson.livescore;
//
//  LiveScore.java
//  LiveScore
//
//  Created by harris wulfson on 7/25/07.
//  Copyright (c) 2007 __MyCompanyName__. All rights reserved.
//
//	For information on setting Java configuration information, including 
//	setting Java properties, refer to the documentation at
//		http://developer.apple.com/techpubs/java/java.html
//


import java.text.AttributedString;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Random;
import java.util.ResourceBundle;

import java.awt.*;
//import java.awt.event.ActionEvent;
//import java.awt.event.KeyEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.*;
import java.awt.geom.Rectangle2D;
import java.net.InetAddress;
//import com.wulfson.livescore.*;

public class LSScore extends JComponent implements ComponentListener
{
	private Font font = new Font("Helvetica", Font.PLAIN, 32); // was Font.ITALIC+Font.BOLD
	protected Font _textFont = new Font("Helvetica", Font.PLAIN, 14);	
	protected ResourceBundle resbundle;
	protected AboutBox aboutBox;
	protected PrefPane prefs;
	//private Application fApplication = Application.getApplication();
	protected Action newAction, openAction, closeAction, saveAction,
			saveAsAction, undoAction, cutAction, copyAction, pasteAction,
			clearAction, selectAllAction;
	//static final JMenuBar mainMenuBar = new JMenuBar();
	//protected JMenu fileMenu, editMenu;
	protected LSPage _page;
	protected LSOscController _oscController;
	protected int _port;
	protected Thread _conductorThread;
	protected double _time; // the current time since start of piece. 
	protected ArrayList<LSTextBox> _textBoxes;
	protected BufferedImage _pageImage;
	protected ArrayList<LSPart> _parts;
	
	public LSScore(int port)
	{
		String address;
		_port = port;
		// The ResourceBundle below contains all of the strings used in this
		// application. ResourceBundles are useful for localizing applications.
		// New localities can be added by adding additional properties files.
		// resbundle = ResourceBundle.getBundle("LiveScorestrings", Locale
		//		.getDefault());
				
		//The component will be its own component listener. good or bad?
		_parts = new ArrayList<LSPart>();
		this.addComponentListener(this); 
		this.addPart(new LSPart(5, LSClef.TREBLE)); //1 part by default
		
		/*
		this.addPart(new LSPart(5, LSClef.TREBLE));
		this.addPart(new LSPart(5, LSClef.TREBLE));
		this.addPart(new LSPart(5, LSClef.TREBLE));
		this.addPart(new LSPart(5, LSClef.BASS));
		this.addPart(new LSPart(5, LSClef.TREBLE8));
		this.addPart(new LSPart(5, LSClef.TREBLE8));
		this.addPart(new LSPart(5, LSClef.PERC));
		this.addPart(new LSPart(5, LSClef.PERC));
		this.addPart(new LSPart(5, LSClef.PERC));
		*/
		
		setSize(800, 600);
		setLocation(200, 100);
		setVisible(true);
		
		//init page
		_page = new LSPage();
		initNotation(this.getSize()); // create staves
				
		try
		{
			_oscController = new LSOscController(this, _port);
		}
		catch(Exception e)
		{
			System.out.println("couldn't start OSC listener.");
		}
		
		_time=0f;

		_textBoxes = new ArrayList<LSTextBox>();
		_conductorThread = new Thread(new LSConductor(this));
		_conductorThread.start();
		
		try
		{
			_page.setTitle("Address: " + InetAddress.getLocalHost().getHostAddress().toString());
		}
		catch(Exception e)
		{
			_page.setTitle("LiveScore: Unknown Host");
		}

	}
	
	public synchronized void initNotation(Dimension d)
	// This was getting called concurently by different threads and screwing up the system ordering!
	// fixed by making synchonized. Very hard to track this down. 
	{
		float topMargin = 24;
		float spacePerPart = 16;
		float extraSpaceBetweenSystems = 4;
		float spacePerSystem;
		
		int systems, idx, nidx;
		Random rand;
		LSSystem system;
		LSSystem lastSystem;
		LSStaff staff;
		LSNote note;
		LSBeamGroup beam;
		Graphics2D g2;
		int staffIdx;

		if(_page == null)
		{
			_page = new LSPage();
		}
		_page.clear();
		_page.setSpace(5f);
		_page.setPageSize(d); 
		_page.setLeftMargin(30);
		_page.setRightMargin(30);

		//Calculate number of systems, based on page size.
		spacePerSystem = spacePerPart * _parts.size() + extraSpaceBetweenSystems;
		systems = (int)((d.height - topMargin) / (spacePerSystem * _page.getSpace()));
		//System.out.println(systems);
		lastSystem = null;
		for (idx = 0; idx < systems; idx++)
		{
			system = _page.newSystem(topMargin + (idx * spacePerSystem));
			if(lastSystem !=null) 
			{
				//used to draw duration beams over the system break. wish this wasn't external. 
				lastSystem.setNextSystem(system);
			}
			
			staffIdx=0;
			for(LSPart aPart : _parts)
			{
				staff = new LSStaff(new LSClef(aPart.getClef()), spacePerPart * staffIdx);
				staff.setLines(aPart.getLines());
				system.addStaff(staff);
				staffIdx++;
			}
			lastSystem = system;
		}
		lastSystem.setNextSystem((LSSystem)_page.getSystems().get(0));
		_pageImage = new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_RGB);
		//this.paintPageImage();
		this.repaint();
	}

	public void paintPageImage() //Was trying out using a bufferedimage, but I abandoned it. Still gets called from cmdNotate
	{
		Graphics2D g2;
		//float space;
		
		g2 = _pageImage.createGraphics();
		//space = _page.getSpace(); //TODO shouldn't the space value be above the page?

		// Gesso it
		g2.setColor(Color.white);
		g2.fillRect(0, 0, this.getSize().width, this.getSize().height);
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setColor(Color.black);
		
		//Paint the page into the buffered image!
		_page.paint(g2);

		g2.dispose();
		
	}
	
	public void paintComponent(Graphics g)
	{
		Graphics2D g2 = (Graphics2D) g;
		float space;
		
		space = _page.getSpace(); //TODO shouldn't the space value be above the page?

		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		
		//Show the page
		//g.drawImage(_pageImage,0,0,null);
		g.setColor(Color.black);
		_page.paint(g);
		
		//Draw conductor bar!
		g.setColor(Color.red);
		this.drawConductorBar(g);
		//this.drawTime(g);
		
		//Draw text boxes
		for(LSTextBox box : _textBoxes)
		{
			box.paint(g);
		}

	}


	public void drawConductorBar(Graphics g)
	{
		Graphics2D g2;
		float space;
		float ext;
		LSPosition position, extraPosition;
		float topYpos, bottomYpos;
		
		g2 = (Graphics2D)g;
		space = _page.getSpace();
		ext=7*space; //length of conductor bar / 2

		//TODO draw two bars for a while when crossing systems
		if(_time > 0)
		{
			position = _page.getPositionForTime(_time);
			extraPosition = _page.getAdditionalConductorPositionForPosition(position);
			topYpos = ((LSSystem) _page.getSystems().get(position.getSystemIndex())).getTopStaff().getYpos();
			bottomYpos = ((LSSystem) _page.getSystems().get(position.getSystemIndex())).getBottomStaff().getYpos();
			g2.setStroke(new BasicStroke(space * 0.14f)); // TODO 0.14
			g2.draw(new Line2D.Float(position.getX(), topYpos - ext,
					position.getX(), bottomYpos + ext));

			//If there is an extra conductor position, then display it. TODO: encapsulate drawing line
			if(extraPosition != null)
			{
				topYpos = ((LSSystem) _page.getSystems().get(extraPosition.getSystemIndex())).getTopStaff().getYpos();
				bottomYpos = ((LSSystem) _page.getSystems().get(extraPosition.getSystemIndex())).getBottomStaff().getYpos();
				g2.setStroke(new BasicStroke(space * 0.14f)); // TODO 0.14
				g2.draw(new Line2D.Float(extraPosition.getX(), topYpos - ext,
						extraPosition.getX(), bottomYpos + ext	));
			}
			
		}		
	}
	public void drawTime(Graphics g)
	{
		int minutes;
		int seconds;
		StringBuffer display;
		
		minutes = (int)(_time / 60);
		seconds = ((int)_time) - (minutes * 60);
		display = new StringBuffer();
		if(minutes > 0)
		{
			display.append(minutes);
		}
		display.append(":");
		if(seconds < 10)
		{
			display.append("0");
		}
		display.append(seconds);
		
		g.setFont(new Font("Helvetica", Font.PLAIN, 14)); //TODO use frame to get font			
		g.setColor(Color.black);
		g.drawString(display.toString(), this.getWidth() - 60, 40);
	}

	//ComponentListener interface methods
    public void componentResized(ComponentEvent evt)
    {
        Component c = (Component)evt.getSource();
        
        //can we check if mouse is down? only want to act if drag is finished.
        // do this by disabling "dynamic layout"
        //System.out.println("Resized!");
        // Get new size
        Dimension newSize = c.getSize();
        this.initNotation(newSize);        
        this.repaint();
    
    }
	public void componentHidden(ComponentEvent arg0)
	{
		// TODO Auto-generated method stub
	}

	public void componentMoved(ComponentEvent arg0)
	{
		// TODO Auto-generated method stub
	}

	public void componentShown(ComponentEvent arg0)
	{
		// TODO Auto-generated method stub	
	}
	
	//Accessors
	public LSPage getPage()
	{
		return _page;
	}

	public void setPage(LSPage page)
	{
		_page = page;
	}
	public ArrayList<LSPart> getParts()
	{
		return _parts;
	}

	public void addPart(LSPart part)
	{
		_parts.add(part);
	}

	public double getTime()
	{
		return _time;
	}
	synchronized public void setTime(double time)
	{
		_time = time;
	}

	public Font getTextFont()
	{
		return _textFont;
	}

	public void setTextFont(Font textFont)
	{
		_textFont = textFont;
	}

	public ArrayList<LSTextBox> getTextBoxes()
	{
		return _textBoxes;
	}

	public void addTextBox(LSTextBox aTextBox)
	{
		_textBoxes.add(aTextBox);
	}

	//methods
	public void clearTextBoxes()
	{
		_textBoxes = new ArrayList<LSTextBox>();
	}

	
	//MAIN
	public static void main(String args[])
	{
		int port;
		
		port = 57110;
		if(args.length > 0)
		{
			port = new Integer(args[0]).intValue();
		}

        final JPanel panel = new JPanel(new BorderLayout());
        LSScore score = new LSScore(port);
        score.setPreferredSize(new Dimension(800,600));
        panel.add(score, BorderLayout.CENTER);
        final JFrame frame = new JFrame("LiveScore");
        frame.getContentPane().add(panel);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
	}

}