package com.wulfson.livescore;
//
//  LSStaff.java
//  LiveScore
//
//  Created by harris wulfson on 7/28/07.
//  Copyright 2007. All rights reserved.
//

//package

import java.util.*;
import java.awt.*;
import java.awt.geom.Line2D;


public class LSStaff
{
	protected LSSystem _system;
	protected float _yoffset;
	protected LSClef _clef;
	protected  ArrayList<LSNote> _notes;
	protected ArrayList<LSBeamGroup> _beamGroups;
	protected int _lines;
	protected ArrayList<LSNote> _noteTrail;
	
	public LSStaff(LSClef clef, float yoffset)
	{
		_notes = new ArrayList<LSNote>();
		_beamGroups = new ArrayList<LSBeamGroup>();
		_clef = clef;
		_clef.setStaff(this);
		//cocoa version allowed a default document-level clef
		_yoffset = yoffset;
		_lines=5;
		_noteTrail = new ArrayList<LSNote>();

	}
	
	public void paint(Graphics g)
	{		
		this.drawStaff(g);
		_clef.paint(g);

		for(LSNote aNote : _notes)
		{
			aNote.paint(g);
		}
		for(LSBeamGroup aBeamGroup : _beamGroups)
		{
			aBeamGroup.paint(g);
		}
	}
	
	public void drawStaff(Graphics g)
	{
		float space;
		float y;
		int lineIndex;
		Graphics2D g2 = (Graphics2D)g;
		
		//g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		space=_system.getPage().getSpace();
        g2.setStroke(new BasicStroke(space * 0.1f));

		y = this.getYpos() - (space * (int)(_lines /2)); //assume 5-line staff
		for(lineIndex=0; lineIndex < _lines; lineIndex++)
		{
			g2.draw(new Line2D.Float(_system.getLeftX(), y, _system.getRightX(), y));
			y=y+space;
		}
		
	}
	
	public void clear()
	{
		_notes = new ArrayList<LSNote>();
		_beamGroups = new ArrayList<LSBeamGroup>();
	}
	
	//Accessors:
	public LSSystem getSystem()
	{
		return _system;
	}

	public void setSystem(LSSystem aSystem)
	{
		_system = aSystem;
	}

	public float getYoffset()
	{
		return _yoffset;
	}

	public void setYoffset(float aYoffset)
	{
		_yoffset = aYoffset;
	}
	
	public float getYpos()
	{
		return _system.getYpos() + (_yoffset * _system.getPage().getSpace());
	}

	public LSClef getClef()
	{
		return _clef;
	}

	public void setClef(LSClef aClef)
	{
		_clef = aClef;
	}

	public ArrayList<LSNote> getNotes()
	{
		return _notes;
	}

	public void addNote(LSNote aNote)
	{
		synchronized(_notes)
		{
			_notes.add(aNote);
		}
	}

	public ArrayList<LSBeamGroup> getBeamGroups()
	{
		return _beamGroups;
	}

	public void addBeamGroup(LSBeamGroup aBeamGroup)
	{
		//bidirectional. copied from cocoa version
		_beamGroups.add(aBeamGroup);
		aBeamGroup.setStaff(this);
	}
	public int getLines()
	{
		return _lines;
	}

	public void setLines(int lines)
	{
		_lines = lines;
	}
	
	public void addToNoteTrail(LSNote note)
	{
		_noteTrail.add(note);
	}
	public ArrayList<LSNote> getNoteTrail()
	{
		return _noteTrail;
	}

	//calculation
	public float getLastNoteEndBeforeX(float x) //returns X of last duration beam end.
	{	
		//This turned out to be complex. 
		float end;
		float max;
		
		max=0f;
		for(LSNote note : _notes)
		{
			end=note.getXpos() + note.getDurationBeamLength();
			if(end > max && (note.getXpos() > x)) //there could be notes later on the staff
			{
				max = end;
			}
		}		
		return max;
	}
}
